@@ -28,17 +28,21 @@ from gtweak.utils import walk_directories, extract_zip_file, make_combo_list_wit
 from gtweak.gsettings import GSettingsSetting, GSettingsMissingError, GSettingsFakeSetting
 from gtweak.gshellwrapper import GnomeShellFactory
 from gtweak.tweakmodel import Tweak, TweakGroup, TWEAK_GROUP_THEME, TWEAK_GROUP_SHELL, TWEAK_SORT_LAST
-from gtweak.widgets import ZipFileChooserButton, GSettingsComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text, UI_BOX_SPACING
+from gtweak.widgets import FileChooserButton, GSettingsComboTweak, GSettingsComboEnumTweak, GSettingsSwitchTweak, adjust_schema_for_overrides, build_label_beside_widget, build_horizontal_sizegroup, build_combo_box_text, UI_BOX_SPACING
+
+_shell = GnomeShellFactory().get_shell()
+_shell_loaded = _shell is not None
 
 class ShowWindowButtons(GSettingsComboTweak):
     def __init__(self, **options):
         GSettingsComboTweak.__init__(self,
-            "org.gnome.shell.overrides",
+            "org.gnome.desktop.wm.preferences",
             "button-layout",
             ((':close', _("Close Only")),
             (':minimize,close', _("Minimize and Close")),
             (':maximize,close', _("Maximize and Close")),
             (':minimize,maximize,close', _("All"))),
+            loaded=_shell_loaded,
             **options)
 
 class ShellThemeTweak(Tweak):
@@ -56,38 +60,39 @@ class ShellThemeTweak(Tweak):
 
         #check the shell is running and the usertheme extension is present
         error = _("Unknown error")
-        try:
-            self._shell = GnomeShellFactory().get_shell()
-        except:
+        self._shell = _shell
+
+        if self._shell is None:
             logging.warning("Shell not running", exc_info=True)
             error = _("Shell not running")
-        try:
-            extensions = self._shell.list_extensions()
-            if ShellThemeTweak.THEME_EXT_NAME in extensions and extensions[ShellThemeTweak.THEME_EXT_NAME]["state"] == 1:
-                #check the correct gsettings key is present
-                try:
-                    if os.path.exists(ShellThemeTweak.THEME_GSETTINGS_DIR):
-                        self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA,
-                                                          schema_dir=ShellThemeTweak.THEME_GSETTINGS_DIR)
-                    else:
-                        self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA)
-                    name = self._settings.get_string(ShellThemeTweak.THEME_GSETTINGS_NAME)
+        else:
+            try:
+                extensions = self._shell.list_extensions()
+                if ShellThemeTweak.THEME_EXT_NAME in extensions and extensions[ShellThemeTweak.THEME_EXT_NAME]["state"] == 1:
+                    #check the correct gsettings key is present
+                    try:
+                        if os.path.exists(ShellThemeTweak.THEME_GSETTINGS_DIR):
+                            self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA,
+                                                              schema_dir=ShellThemeTweak.THEME_GSETTINGS_DIR)
+                        else:
+                            self._settings = GSettingsSetting(ShellThemeTweak.THEME_GSETTINGS_SCHEMA)
+                        name = self._settings.get_string(ShellThemeTweak.THEME_GSETTINGS_NAME)
 
-                    ext = extensions[ShellThemeTweak.THEME_EXT_NAME]
-                    logging.debug("Shell user-theme extension\n%s" % pprint.pformat(ext))
+                        ext = extensions[ShellThemeTweak.THEME_EXT_NAME]
+                        logging.debug("Shell user-theme extension\n%s" % pprint.pformat(ext))
 
-                    error = None
-                except:
-                    logging.warning(
-                        "Could not find user-theme extension in %s" % ','.join(extensions.keys()),
-                        exc_info=True)
-                    error = _("Shell user-theme extension incorrectly installed")
+                        error = None
+                    except:
+                        logging.warning(
+                            "Could not find user-theme extension in %s" % ','.join(extensions.keys()),
+                            exc_info=True)
+                        error = _("Shell user-theme extension incorrectly installed")
 
-            else:
-                error = _("Shell user-theme extension not enabled")
-        except Exception, e:
-            logging.warning("Could not list shell extensions", exc_info=True)
-            error = _("Could not list shell extensions")
+                else:
+                    error = _("Shell user-theme extension not enabled")
+            except Exception, e:
+                logging.warning("Could not list shell extensions", exc_info=True)
+                error = _("Could not list shell extensions")
 
         if error:
             cb = build_combo_box_text(None)
@@ -119,7 +124,10 @@ class ShellThemeTweak(Tweak):
             self._combo = cb
 
             #a filechooser to install new themes
-            chooser = ZipFileChooserButton(_("Select a theme"))
+            chooser = FileChooserButton(
+                        _("Select a theme"),
+                        True,
+                        ["application/zip"])
             chooser.connect("file-set", self._on_file_set)
 
             self.widget = build_label_beside_widget(self.name, chooser, cb)
@@ -198,10 +206,11 @@ class StaticWorkspaceTweak(Tweak):
     NUM_WORKSPACES_SCHEMA = "org.gnome.desktop.wm.preferences"
     NUM_WORKSPACES_KEY = "num-workspaces"
 
-    DYNAMIC_SCHEMA = "org.gnome.shell.overrides"
     DYNAMIC_KEY = "dynamic-workspaces"
+    DYNAMIC_SCHEMA = "org.gnome.mutter"
 
     def __init__(self, **options):
+        schema = adjust_schema_for_overrides(self.DYNAMIC_SCHEMA, self.DYNAMIC_KEY, options)
         Tweak.__init__(self, _("Dynamic workspaces"), _("Disable gnome-shell dynamic workspace management, use static workspaces"), **options)
 
         try:
@@ -211,7 +220,7 @@ class StaticWorkspaceTweak(Tweak):
             nwsettings = GSettingsFakeSetting()
 
         try:
-            dsettings = GSettingsSetting(self.DYNAMIC_SCHEMA, **options)
+            dsettings = GSettingsSetting(schema, **options)
         except GSettingsMissingError:
             self.loaded = False
             dsettings = GSettingsFakeSetting()
@@ -236,7 +245,7 @@ class StaticWorkspaceTweak(Tweak):
 sg = build_horizontal_sizegroup()
 
 TWEAKS = (
-    ShellThemeTweak(group_name=TWEAK_GROUP_THEME),
+    ShellThemeTweak(group_name=TWEAK_GROUP_THEME, loaded=_shell_loaded),
 )
 
 TWEAK_GROUPS = (
@@ -244,12 +253,13 @@ TWEAK_GROUPS = (
             TWEAK_GROUP_SHELL,
             GSettingsSwitchTweak("org.gnome.desktop.interface", "clock-show-date", schema_filename="org.gnome.desktop.interface.gschema.xml"),
             GSettingsSwitchTweak("org.gnome.desktop.interface", "clock-show-seconds", schema_filename="org.gnome.desktop.interface.gschema.xml"),
-            GSettingsSwitchTweak("org.gnome.shell.calendar", "show-weekdate", schema_filename="org.gnome.shell.gschema.xml"),
-            ShowWindowButtons(schema_filename="org.gnome.shell.gschema.xml", size_group=sg),
+            GSettingsSwitchTweak("org.gnome.shell.calendar", "show-weekdate", schema_filename="org.gnome.shell.gschema.xml", loaded=_shell_loaded),
+            ShowWindowButtons(size_group=sg),
             GSettingsSwitchTweak("org.gnome.settings-daemon.plugins.power", "lid-close-suspend-with-external-monitor"),
             GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "lid-close-battery-action", size_group=sg),
             GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "lid-close-ac-action", size_group=sg),
             GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.power", "button-power", size_group=sg),
-            GSettingsSwitchTweak("org.gnome.shell.overrides", "workspaces-only-on-primary"),
-            StaticWorkspaceTweak(size_group=sg)),
+            GSettingsComboEnumTweak("org.gnome.settings-daemon.plugins.xrandr", "default-monitors-setup", size_group=sg),
+            GSettingsSwitchTweak("org.gnome.mutter", "workspaces-only-on-primary", schema_filename="org.gnome.shell.gschema.xml", loaded=_shell_loaded),
+            StaticWorkspaceTweak(size_group=sg, loaded=_shell_loaded)),
 )
